//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Main.h"
#include "ecc.h"
#include "bin2str.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMain *frmMain;

uint8_t p_publicKey[ECC_BYTES+1];
uint8_t p_privateKey[ECC_BYTES];
unsigned char paddedHash[ECC_BYTES];
uint8_t p_signature[ECC_BYTES*2];

//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::btnMakeKeysClick(TObject *Sender)
{
	if (ecc_make_key(p_publicKey, p_privateKey) == 0)
	{
		ShowMessage("Failed to generate keys!");
		return;
	}
	AnsiString tmp;

	tmp = "";
	for (int i=0; i<sizeof(p_privateKey); i++)
	{
		tmp.cat_printf("%02X", p_privateKey[i]);
	}
	edKeyPrivate->Text = tmp;

	tmp = "";
	for (int i=0; i<sizeof(p_publicKey); i++)
	{
		tmp.cat_printf("%02X", p_publicKey[i]);
	}
	edKeyPublic->Text = tmp;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::btnSignClick(TObject *Sender)
{
	std::string publicK = HexStringToBuf(edKeyPublic->Text.c_str());
	if (publicK.size() != sizeof(p_publicKey))
	{
		ShowMessage("Incorrect public key length!");
		return;
	}
	memcpy(p_publicKey, publicK.data(), publicK.size());

	std::string privK = HexStringToBuf(edKeyPrivate->Text.c_str());
	if (privK.size() != sizeof(p_privateKey))
	{
		ShowMessage("Incorrect private key length!");
		return;
	}
	memcpy(p_privateKey, privK.data(), privK.size());

	std::string hash = HexStringToBuf(edHash->Text.c_str());
	memset(paddedHash, 0, sizeof(paddedHash));
	memcpy(paddedHash, hash.data(), hash.size());
	if (0 == ecdsa_sign(p_privateKey, paddedHash, p_signature))
	{
    	ShowMessage("Failed!");
	}
	else
	{
		edSignature->Text = BufToHexString(p_signature, sizeof(p_signature)).c_str();
	}
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::btnVerifyClick(TObject *Sender)
{
	std::string publicK = HexStringToBuf(edKeyPublic->Text.c_str());
	if (publicK.size() != sizeof(p_publicKey))
	{
		ShowMessage("Incorrect public key length!");
		return;
	}
	memcpy(p_publicKey, publicK.data(), publicK.size());

	std::string privK = HexStringToBuf(edKeyPrivate->Text.c_str());
	if (privK.size() != sizeof(p_privateKey))
	{
		ShowMessage("Incorrect private key length!");
		return;
	}
	memcpy(p_privateKey, privK.data(), privK.size());

	std::string hash = HexStringToBuf(edHash->Text.c_str());
	memset(paddedHash, 0, sizeof(paddedHash));
	memcpy(paddedHash, hash.data(), hash.size());

	std::string signature = HexStringToBuf(edSignature->Text.c_str());
	if (signature.size() != sizeof(p_signature))
	{
		ShowMessage("Incorrect signature length!");
		return;
	}
	memcpy(p_signature, signature.data(), signature.size());

	if (0 == ecdsa_verify(p_publicKey, paddedHash, p_signature))
	{
		ShowMessage("Failed!");
	}
	else
	{
		ShowMessage("OK!");
	}
}
//---------------------------------------------------------------------------


